# -*- coding: utf-8 -*-
"""
	Coins
    ~~~~~

    :copyright: (c) 2013 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

__version__ = '0.1'

from .wallets import BitcoinWallet, LitecoinWallet, NamecoinWallet, \
	PeercoinWallet, PrimecoinWallet



# -*- coding: utf-8 -*-
"""
    Coins
    ~~~~~
    
    :copyright: (c) 2013 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import ecdsa
import hashlib
import binascii
from utils import dev_random_entropy, random_secret_exponent, is_hex, \
    binary_hash160, base58check_encode

class BitcoinWallet():
    curve = ecdsa.curves.SECP256k1
    hash_function = hashlib.sha256
    version_bytes = {
        'pubkey_hash': 0,
        'private_key': 0+128,
    }

    def __init__(self, hex_private_key=None):
        """ Takes in a private key/secret exponent as a 64-character
        hex string.
        """
        if not hex_private_key:
            hex_private_key = random_secret_exponent()

        if not (len(hex_private_key) == 64 and is_hex(hex_private_key)):
            raise Exception('Invalid private key. Must be a 64-char hex string.')

        int_secret_exponent = int(hex_private_key, 16)

        self.private_key = ecdsa.keys.SigningKey.from_secret_exponent(
            int_secret_exponent, self.curve, self.hash_function
        )

    def bin_private_key(self):
        return self.private_key.to_string()

    def hex_private_key(self):
        return binascii.hexlify(self.bin_private_key())

    def bin_public_key(self):
        public_key = self.private_key.get_verifying_key()
        return '\x04' + public_key.to_string()

    def hex_public_key(self):
        return binascii.hexlify(self.bin_public_key())

    def bin_hash_160(self):
        return binary_hash160(self.bin_public_key())

    def hex_hash_160(self):
        return binascii.hexlify(self.bin_hash_160())

    def wif_private_key(self):
        version_byte = self.version_bytes['private_key']
        return base58check_encode(self.bin_private_key(), version_byte=version_byte)

    def address(self):
        version_byte = self.version_bytes['pubkey_hash']
        return base58check_encode(self.bin_hash_160(), version_byte=version_byte)

class LitecoinWallet(BitcoinWallet):
    version_bytes = {
        'pubkey_hash': 48,
        'private_key': 48+128,
    }

class NamecoinWallet(BitcoinWallet):
    version_bytes = {
        'pubkey_hash': 52,
        'private_key': 52+128,
    }

class PeercoinWallet(BitcoinWallet):
    version_bytes = {
        'pubkey_hash': 55,
        'private_key': 55+128,
    }

class PrimecoinWallet(BitcoinWallet):
    version_bytes = {
        'pubkey_hash': 23,
        'private_key': 23+128,
    }




# -*- coding: utf-8 -*-
"""
    Coins
    ~~~~~
    
    :copyright: (c) 2013 by Halfmoon Labs
    :license: MIT, see LICENSE for more details.
"""

import re
import random
import hashlib
import binascii

def dev_random_entropy(numbytes):
    return open("/dev/random", "rb").read(numbytes)

def dev_urandom_entropy(numbytes):
    return open("/dev/urandom", "rb").read(numbytes)

def random_secret_exponent():
    """ Generates a random secret exponent and returns it as a hex string. """
    return binascii.hexlify(dev_random_entropy(32))

def is_hex(string):
    try:
        int(string, 16)
    except ValueError:
        return False
    else:
        return True

def int_to_hex(i):
    return hex(i).rstrip("L").lstrip("0x") or "0"

def int_to_string(integer, keyspace_chars):
    """ Turn a positive integer into a string. """
    if not integer > 0:
        raise ValueError('integer must be > 0')
    output = ""
    while integer > 0:
        integer, digit = divmod(integer, len(keyspace_chars))
        output += keyspace_chars[digit]
    return output[::-1]

def string_to_int(string, keyspace_chars):
    """ Turn a string into a positive integer. """
    output = 0
    for char in string:
        output = output * len(keyspace_chars) + keyspace_chars.index(char)
    return output

def change_keyspace(string, original_keyspace, target_keyspace):
    """ Convert a string from one keyspace to another. """
    assert isinstance(string, str)
    intermediate_integer = string_to_int(string, original_keyspace)
    output_string = int_to_string(intermediate_integer, target_keyspace)
    return output_string

def binary_sha256(s):
    return hashlib.sha256(s).digest()

def binary_hash160(s):
    return hashlib.new('ripemd160', binary_sha256(s)).digest()

def base58check_encode(s, version_byte=0):
    HEX_KEYSPACE = "0123456789abcdef"
    B58_KEYSPACE = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
    
    s = chr(int(version_byte)) + s
    leading_bytes = len(re.match('^\x00*', s).group(0))
    checksum = binary_sha256(binary_sha256(s))[:4]
    hex_s = binascii.hexlify(s + checksum)
    
    return '1' * leading_bytes + change_keyspace(hex_s, HEX_KEYSPACE, B58_KEYSPACE)




"""
Coins
==============

"""

from setuptools import setup

setup(
	name='Coins',
	version='0.1.1',
	url='https://github.com/halfmoonlabs/coins',
	license='MIT',
	author='Halfmoon Labs',
	author_email='hello@halfmoon.io',
	description='Wallet interface for popular cryptocurrencies, including Bitcoin, Litecoin, Namecoin, Peercoin, and Primecoin.',
	packages=[
		'coins',
	],
	zip_safe=False,
	install_requires=[
		'ecdsa>=0.10',
	],
	classifiers=[
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Programming Language :: Python',
	],
)
